# Generated by Django 5.2.1 on 2025-06-11 10:40

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('base', '0005_initial'),
        ('invoices', '0004_invoicelineitem_charge_item_reference'),
    ]

    operations = [
        migrations.CreateModel(
            name='InvoiceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_reference_identifier', to='base.identifier')),
                ('invoice', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_reference_invoice', to='invoices.invoice')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""payment reconciliations URL Configuration."""

from django.urls import path

from dfhir.paymentreconciliations import views

app_name = "paymentreconciliations"

urlpatterns = [
    path("paymentreconciliations/", views.PaymentReconciliationListView.as_view()),
    path(
        "paymentreconciliations/<int:pk>/",
        views.PaymentReconciliationDetailView.as_view(),
    ),
]



"""Person model."""

from django.db import models

from dfhir.base.choices import GenderChoices
from dfhir.base.models import (
    Address,
    Attachment,
    BaseReference,
    CodeableConcept,
    Communication,
    ContactPoint,
    HumanName,
    Identifier,
    OrganizationReference,
    TimeStampedModel,
)

from . import choices


class PersonLinkTargetReference(BaseReference):
    """Model definition for PersonLinkTargetReference."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="person_link_target_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    person = models.ForeignKey(
        "Person",
        related_name="person_link_target_reference_person",
        on_delete=models.CASCADE,
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        related_name="person_link_target_reference_patient",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        related_name="person_link_target_reference_practitioner",
        on_delete=models.CASCADE,
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        related_name="person_link_target_reference_related_person",
        on_delete=models.CASCADE,
        null=True,
    )


class PersonLink(TimeStampedModel):
    """Model definition for PersonLink."""

    target = models.ForeignKey(
        PersonLinkTargetReference,
        related_name="person_link_target",
        on_delete=models.CASCADE,
    )
    assurance = models.CharField(
        max_length=255, choices=choices.LinkAssuranceChoices.choices, null=True
    )


class Person(TimeStampedModel):
    """Model definition for Person."""

    identifier = models.ManyToManyField(
        Identifier, related_name="persons_identifier", blank=True
    )
    active = models.BooleanField(default=True)
    name = models.ManyToManyField(HumanName, related_name="persons_name", blank=True)
    telecom = models.ManyToManyField(
        ContactPoint, related_name="persons_telecom", blank=True
    )
    gender = models.CharField(max_length=255, choices=GenderChoices.choices, null=True)
    birth_date = models.DateField(null=True)
    deceased_boolean = models.BooleanField(null=True)
    deceased_date_time = models.DateTimeField(null=True)
    address = models.ManyToManyField(
        Address, related_name="persons_address", blank=True
    )
    marital_status = models.ForeignKey(
        CodeableConcept,
        related_name="persons_marital_status",
        on_delete=models.CASCADE,
        null=True,
    )
    photo = models.ManyToManyField(Attachment, related_name="persons_photo", blank=True)
    communication = models.ManyToManyField(
        Communication, related_name="persons_communication", blank=True
    )
    managing_organization = models.ForeignKey(
        OrganizationReference,
        related_name="persons_managing_organization",
        on_delete=models.CASCADE,
        null=True,
    )
    link = models.ManyToManyField(PersonLink, related_name="persons_link", blank=True)


class PersonReference(BaseReference):
    """Model definition for PersonReference."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="person_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    person = models.ForeignKey(
        Person,
        related_name="person_reference_person",
        on_delete=models.CASCADE,
        null=True,
    )



"""Goals models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Duration,
    Identifier,
    Quantity,
    Range,
    Ratio,
    TimeStampedModel,
)

from . import choices


class GoalSubjectReference(BaseReference):
    """Goal Subject Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="goal_subject_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="goal_subject_reference_patient",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        related_name="goal_subject_reference_group",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="goal_subject_reference_organization",
        null=True,
    )


class GoalAcceptanceParticipantReference(BaseReference):
    """Goal Acceptance Participant Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_participant_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_participant_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_participant_reference_practitioner",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_participant_reference_related_person",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_participant_reference_practitioner_role",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_participant_reference_care_team",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_participant_reference_organization",
        null=True,
    )


class GoalAcceptance(TimeStampedModel):
    """Goal Acceptance model."""

    participant = models.ForeignKey(
        GoalAcceptanceParticipantReference,
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_participant",
        null=True,
    )
    status = models.CharField(
        max_length=255, choices=choices.GoalAcceptanceStatusChoices.choices
    )
    priority = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="goal_acceptance_priority",
        null=True,
    )


class GoalTarget(TimeStampedModel):
    """Goal Target model."""

    measure = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="goal_target_measure",
        null=True,
    )
    detail_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="goal_target_detail_quantity",
        null=True,
    )
    detail_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        related_name="goal_target_detail_range",
        null=True,
    )
    detail_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="goal_target_detail_codeable_concept",
        null=True,
    )
    detail_string = models.CharField(max_length=255, null=True)
    detail_boolean = models.BooleanField(default=False)
    detail_integer = models.IntegerField(null=True)
    detail_ratio = models.ForeignKey(
        Ratio,
        on_delete=models.DO_NOTHING,
        related_name="goal_target_detail_ratio",
        null=True,
    )
    due_date = models.DateField(null=True)
    due_duration = models.ForeignKey(
        Duration,
        on_delete=models.DO_NOTHING,
        related_name="goal_target_due_duration",
        null=True,
    )


class GoalSourceReference(BaseReference):
    """Goal source reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="goal_source_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="goal_source_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="goal_source_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="goal_source_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="goal_source_reference_related_person",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.CASCADE,
        related_name="goal_source_reference_care_team",
        null=True,
    )


class GoalAddressesReference(BaseReference):
    """Goal addresses reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_identifier",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_condition",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_observation",
        null=True,
    )

    medication_statement = models.ForeignKey(
        "medicationstatements.MedicationStatement",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_medication_statement",
        null=True,
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_medication_request",
        null=True,
    )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_nutrition_order",
        null=True,
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_service_request",
        null=True,
    )
    risk_assessment = models.ForeignKey(
        "riskassessments.RiskAssessment",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_risk_assessment",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_procedure",
        null=True,
    )
    nutrition_intake = models.ForeignKey(
        "nutritionintakes.NutritionIntake",
        on_delete=models.CASCADE,
        related_name="goal_addresses_reference_nutrition_intake",
        null=True,
    )


class Goal(TimeStampedModel):
    """Goal model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="goal_identifier",
        blank=True,
    )
    lifecycle_status = models.CharField(
        max_length=255, choices=choices.GoalLifecycleStatusChoices.choices
    )
    achievement_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="goal_achievement_status",
        null=True,
    )
    category = models.ManyToManyField(
        CodeableConcept,
        related_name="goal_category",
        blank=True,
    )
    continuous = models.BooleanField(default=False)
    priority = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="goal_priority",
        null=True,
    )
    description = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="goal_description",
        null=True,
    )
    subject = models.ForeignKey(
        GoalSubjectReference,
        on_delete=models.DO_NOTHING,
        related_name="goal_subject",
        null=True,
    )
    start_date = models.DateField(null=True)
    start_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="goal_start_code",
        null=True,
    )
    acceptance = models.ManyToManyField(
        GoalAcceptance,
        related_name="goal_acceptance",
        blank=True,
    )
    target = models.ManyToManyField(
        GoalTarget,
        related_name="goal_target",
        blank=True,
    )
    status_date = models.DateField(null=True)
    status_reason = models.ManyToManyField(
        CodeableConcept,
        related_name="goal_status_code",
        blank=True,
    )
    source = models.ForeignKey(
        GoalSourceReference,
        on_delete=models.DO_NOTHING,
        related_name="goal_source",
        null=True,
    )
    addresses = models.ManyToManyField(
        GoalAddressesReference,
        related_name="goal_addresses",
        blank=True,
    )
    note = models.ManyToManyField(
        Annotation,
        related_name="goal_note",
        blank=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='RelatedPerson',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('gender', models.CharField(blank=True, choices=[('male', 'male'), ('female', 'female'), ('other', 'other'), ('unknown', 'unknown')], max_length=20, null=True)),
                ('birth_date', models.DateField(null=True)),
                ('address', models.ManyToManyField(blank=True, related_name='relatedpersons_address', to='base.address')),
                ('communication', models.ManyToManyField(blank=True, related_name='relatedpersons_communication', to='base.communication')),
                ('identifier', models.ManyToManyField(blank=True, related_name='relatedpersons_identifier', to='base.identifier')),
                ('name', models.ManyToManyField(blank=True, related_name='relatedpersons_name', to='base.humanname')),
                ('patient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='relatedpersons_patient', to='patients.patientreference')),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='relatedpersons_period', to='base.period')),
                ('photo', models.ManyToManyField(blank=True, related_name='relatedpersons_photo', to='base.attachment')),
                ('relationship', models.ManyToManyField(blank=True, related_name='relatedpersons_relationship', to='base.codeableconcept')),
                ('role', models.ManyToManyField(blank=True, related_name='relatedpersons_role', to='base.codeableconcept')),
                ('telecom', models.ManyToManyField(blank=True, related_name='relatedpersons_telecom', to='base.contactpoint')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RelatedPersonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_person_reference_identifier', to='base.identifier')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_person_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('allergyintolerances', '0004_initial'),
        ('base', '0004_initial'),
        ('observations', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='allergyintoleranceasserterreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_asserter_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='allergyintoleranceasserterreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_asserter_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='allergyintoleranceasserterreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_asserter_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='asserter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_asserter', to='allergyintolerances.allergyintoleranceasserterreference'),
        ),
        migrations.AddField(
            model_name='allergyintolerancereaction',
            name='exposure_route',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_reaction_exposure_route', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='allergyintolerancereaction',
            name='manifestation',
            field=models.ManyToManyField(blank=True, related_name='allergy_intolerance_reaction_manifestation', to='observations.observationcodeablereference'),
        ),
        migrations.AddField(
            model_name='allergyintolerancereaction',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='allergy_intolerance_reaction_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='allergyintolerancereaction',
            name='substance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_reaction_substance', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='reaction',
            field=models.ManyToManyField(blank=True, related_name='allergy_intolerance_reaction', to='allergyintolerances.allergyintolerancereaction'),
        ),
        migrations.AddField(
            model_name='allergyintolerancerecorderreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_recorder_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='allergyintolerancerecorderreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_recorder_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='allergyintolerancerecorderreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_recorder_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='allergyintolerancerecorderreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_recorder_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='allergyintolerancerecorderreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_recorder_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='allergyintolerancerecorderreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_recorder_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='recorder',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_recorder', to='allergyintolerances.allergyintolerancerecorderreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('appointments', '0002_initial'),
        ('base', '0003_initial'),
        ('careplans', '0003_initial'),
        ('endpoints', '0001_initial'),
        ('imagingselections', '0001_initial'),
        ('imagingstudies', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='imagingselection',
            name='derived_from',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='image_selection_derived_from', to='imagingstudies.imagingstudyreference'),
        ),
        migrations.AddField(
            model_name='imagingselection',
            name='endpoint',
            field=models.ManyToManyField(blank=True, related_name='imaging_selection_endpoint', to='endpoints.endpointreference'),
        ),
        migrations.AddField(
            model_name='imagingselection',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='image_selection_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='imagingselectionbasedonreference',
            name='appointment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='image_selection_based_on_reference_appointment', to='appointments.appointment'),
        ),
        migrations.AddField(
            model_name='imagingselectionbasedonreference',
            name='care_plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='image_selection_based_on_reference_care_plan', to='careplans.careplan'),
        ),
        migrations.AddField(
            model_name='imagingselectionbasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='image_selection_based_on_reference_identifier', to='base.identifier'),
        ),
    ]



"""Patients app configuration."""

import contextlib

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class PatientsConfig(AppConfig):
    """Patients app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.patients"
    verbose_name = _("Patients")

    def ready(self):
        """Import signals."""
        with contextlib.suppress(ImportError):
            import dfhir.patients.signals  # noqa: F401



"""charge item views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.chargeitems.models import ChargeItem
from dfhir.chargeitems.serializers import ChargeItemSerializer


class ChargeItemListView(APIView):
    """ChargeItem list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ChargeItemSerializer(many=True)})
    def get(self, request):
        """Get all charge items."""
        charge_items = ChargeItem.objects.all()
        serializer = ChargeItemSerializer(charge_items, many=True)
        return Response(serializer.data)

    @extend_schema(request=ChargeItemSerializer, responses={201: ChargeItemSerializer})
    def post(self, request):
        """Create a charge item."""
        serializer = ChargeItemSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ChargeItemDetailView(APIView):
    """ChargeItem detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a charge item object."""
        try:
            return ChargeItem.objects.get(pk=pk)
        except ChargeItem.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ChargeItemSerializer})
    def get(self, request, pk):
        """Get a charge item."""
        charge_item = self.get_object(pk)
        serializer = ChargeItemSerializer(charge_item)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=ChargeItemSerializer, responses={200: ChargeItemSerializer})
    def patch(self, request, pk):
        """Update a charge item."""
        charge_item = self.get_object(pk)
        serializer = ChargeItemSerializer(charge_item, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk):
        """Delete a charge item."""
        charge_item = self.get_object(pk)
        charge_item.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
        ('locations', '0001_initial'),
        ('organizations', '0001_initial'),
        ('provenances', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContractContentDefinition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('publication_date', models.DateTimeField(null=True)),
                ('publication_status', models.CharField(choices=[('amended', 'Amended'), ('appended', 'Appended'), ('cancelled', 'Cancelled'), ('disputed', 'Disputed'), ('entered-in-error', 'Entered in Error'), ('executed', 'Executed'), ('executable', 'Executable'), ('negotiable', 'Negotiable'), ('offered', 'Offered'), ('rejected', 'Rejected'), ('renewed', 'Renewed'), ('revoked', 'Revoked'), ('resolved', 'Resolved'), ('terminated', 'Terminated')], max_length=255, null=True)),
                ('copyright', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractContentDefinitionPublisherReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractFriendly',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractFriendlyContentReferenceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractLegal',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractLegalContentReferenceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractLegallyBindingReferenceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractRule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractSigner',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractSignerPartyReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTerm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('issued', models.DateTimeField(null=True)),
                ('text', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermAction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('do_not_perform', models.BooleanField(default=False)),
                ('link_id', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('context_link_id', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('occurrence_date_time', models.DateTimeField(null=True)),
                ('requester_link_id', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('performer_link_id', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('reason_link_id', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('security_label_number', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermActionPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermActionReasonCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermActionReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermActionRequesterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermActionSubject',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermActionSubjectReferenceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermAsset',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('condition', models.TextField(null=True)),
                ('text', models.TextField(null=True)),
                ('link_id', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('security_label_number', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermAssetContext',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('text', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermAssetValuedItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('effective_time', models.DateTimeField(null=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('points', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('payment', models.CharField(max_length=255, null=True)),
                ('payment_date', models.DateTimeField(null=True)),
                ('link_id', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('security_label_number', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermAssetValuedItemRecipientReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermAssetValuedItemResponsibleReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermOffer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('text', models.TextField(null=True)),
                ('link_id', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('security_label_number', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermOfferAnswer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_decimal', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('value_integer', models.IntegerField(null=True)),
                ('value_date', models.DateField(null=True)),
                ('value_date_time', models.DateTimeField(null=True)),
                ('value_time', models.TimeField(null=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('value_uri', models.URLField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermOfferParty',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermOfferPartyReferenceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractTermSecurityLabel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('number', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Contract',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('url', models.URLField(null=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('status', models.CharField(choices=[('amended', 'Amended'), ('appended', 'Appended'), ('cancelled', 'Cancelled'), ('disputed', 'Disputed'), ('entered-in-error', 'Entered in Error'), ('executed', 'Executed'), ('executable', 'Executable'), ('negotiable', 'Negotiable'), ('offered', 'Offered'), ('rejected', 'Rejected'), ('renewed', 'Renewed'), ('revoked', 'Revoked'), ('resolved', 'Resolved'), ('terminated', 'Terminated')], max_length=255, null=True)),
                ('instantiates_uri', models.URLField(null=True)),
                ('issued', models.DateTimeField(null=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('subtitle', models.CharField(max_length=255, null=True)),
                ('alias', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('applies', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_applies', to='base.period')),
                ('authority', models.ManyToManyField(blank=True, related_name='contract_authority', to='base.organizationreference')),
                ('content_derivative', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_derivative', to='base.codeableconcept')),
                ('domain', models.ManyToManyField(blank=True, related_name='contract_domain', to='locations.locationreference')),
                ('expiration_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_expiration_type', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='contract_identifier', to='base.identifier')),
                ('legal_state', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legal_state', to='base.codeableconcept')),
                ('legally_binding_attachment', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legally_binding_attachment', to='base.attachment')),
                ('relevant_history', models.ManyToManyField(blank=True, related_name='contract_relevant_history', to='provenances.provenancereference')),
                ('scope', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_scope', to='base.codeableconcept')),
                ('site', models.ManyToManyField(blank=True, related_name='contract_site', to='locations.locationreference')),
                ('sub_type', models.ManyToManyField(blank=True, related_name='contract_sub_type', to='base.codeableconcept')),
                ('supporting_info', models.ManyToManyField(blank=True, related_name='contract_supporting_info', to='base.reference')),
                ('topic_codeable_concept', models.ManyToManyField(blank=True, related_name='contract_topic_codeable_concept', to='base.codeableconcept')),
                ('topic_reference', models.ManyToManyField(blank=True, related_name='contract_topic_reference', to='base.reference')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContractAuthorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_author_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_author_reference_organization', to='organizations.organization')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Clinical impressions urls."""

from django.urls import path

from . import views

app_name = "clinicalimpressions"

urlpatterns = [
    path(
        "clincalimpressions/", views.ClinicalImpressionListView.as_view(), name="create"
    ),
    path(
        "clincalimpressions/<int:pk>/",
        views.ClinicalImpressionsDetailView.as_view(),
        name="detail",
    ),
]



# Generated by Django 5.2 on 2025-05-05 14:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allergyintolerances', '0005_initial'),
        ('base', '0005_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllergyIntoleranceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('allergy_intolerance', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='allergy_intolerance_reference_allergy_intolerance', to='allergyintolerances.allergyintolerance')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='allergy_intolerance_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='extendedcontactdetail',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='extended_contact_detail_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='extendedcontactdetail',
            name='purpose',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='extended_contact_detail_purpose', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='extendedcontactdetail',
            name='telecom',
            field=models.ManyToManyField(blank=True, related_name='extended_contact_detail_telecom', to='base.contactpoint'),
        ),
        migrations.AddField(
            model_name='extendedcontactdetail',
            name='name',
            field=models.ManyToManyField(blank=True, related_name='extended_contact_detail_name', to='base.humanname'),
        ),
        migrations.AddField(
            model_name='identifier',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='monetarycomponent',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='monetory_component_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='monetarycomponent',
            name='amount',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='monetory_component_amount', to='base.money'),
        ),
        migrations.AddField(
            model_name='availability',
            name='not_available_time',
            field=models.ManyToManyField(blank=True, related_name='availability_not_available_time', to='base.notavailabletime'),
        ),
        migrations.AddField(
            model_name='organizationreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='organization_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='organizationreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='organization_reference', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='identifier',
            name='assigner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='identifier_for_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='payload',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='payload_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='notavailabletime',
            name='during',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='not_available_time_during', to='base.period'),
        ),
        migrations.AddField(
            model_name='identifier',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='identifier', to='base.period'),
        ),
        migrations.AddField(
            model_name='humanname',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='human_name_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='extendedcontactdetail',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='extended_contact_detail_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='contactpoint',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='contact_point_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='availability',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='availability_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='address',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='address_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='productshelflife',
            name='period_duration',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='product_shelf_life_period_duration', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='productshelflife',
            name='special_precautions_for_storage',
            field=models.ManyToManyField(blank=True, related_name='product_shelf_life_special_precautions_for_storage', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='productshelflife',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='product_shelf_life_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='qualification',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='qualification_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='qualification',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='qualification_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='qualification',
            name='issuer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='qualification_issuer', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='qualification',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='qualification_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='range',
            name='high',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='range_high', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='range',
            name='low',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='range_low', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='ratio',
            name='denominator',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='ratio_denominator', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='ratio',
            name='numerator',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='ratio_numerator', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='reference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.identifier'),
        ),
        migrations.AddField(
            model_name='codeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='codeable_reference_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='annotation',
            name='author_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='annotation_author_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='relatedartifact',
            name='classifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='related_artifact_classifier', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='relatedartifact',
            name='document',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='related_artifact_document', to='base.attachment'),
        ),
        migrations.AddField(
            model_name='relatedartifact',
            name='resource_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='related_artifact_resource', to='base.reference'),
        ),
        migrations.AddField(
            model_name='relativetime',
            name='context_code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='relative_time_context_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='relativetime',
            name='context_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='relative_time_context_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='relativetime',
            name='offset_duration',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='relative_time_offset_duration', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='relativetime',
            name='offset_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='relative_time_offset_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='repeat',
            name='bounds_duration',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='repeat_bounds_duration', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='repeat',
            name='bounds_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='repeat_bounds_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='repeat',
            name='bounds_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='repeat_bounds_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='repeat',
            name='day_of_week',
            field=models.ManyToManyField(blank=True, related_name='repeat_day_of_week', to='base.coding'),
        ),
        migrations.AddField(
            model_name='repeat',
            name='when',
            field=models.ManyToManyField(blank=True, related_name='repeat_when', to='base.coding'),
        ),
        migrations.AddField(
            model_name='signature',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='signature_type', to='base.coding'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('bodystructures', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentReferenceAttesterPartyReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceAuthorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceContent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceContentProfile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_uri', models.URLField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceContextReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceRelatesTo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceAttester',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('time', models.DateTimeField(null=True)),
                ('mode', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_attesters_mode', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('status', models.CharField(choices=[('current', 'Current'), ('suppressed', 'Suppressed'), ('entered-in-error', 'Entered in error')], max_length=255, null=True)),
                ('doc_status', models.CharField(choices=[('registered', 'Registered'), ('partial', 'Partial'), ('preliminary', 'Preliminary'), ('final', 'Final'), ('amended', 'Amended'), ('corrected', 'Corrected'), ('appended', 'Appended'), ('cancelled', 'Cancelled'), ('entered-in-error', 'Entered in error'), ('deprecated', 'Deprecated'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('date', models.DateTimeField(null=True)),
                ('description', models.TextField(null=True)),
                ('body_site', models.ManyToManyField(blank=True, related_name='document_reference_body_sites', to='bodystructures.bodystructurecodeablereference')),
                ('category', models.ManyToManyField(blank=True, related_name='document_reference_categories', to='base.codeableconcept')),
                ('custodian', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_custodian', to='base.organizationreference')),
                ('event', models.ManyToManyField(blank=True, related_name='document_reference_events', to='base.codeablereference')),
                ('facility_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_facility_type', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='document_reference_identifier', to='base.identifier')),
                ('modality', models.ManyToManyField(blank=True, related_name='document_reference_modalities', to='base.codeableconcept')),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_source_patient_infos', to='base.period')),
                ('practice_setting', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_practice_setting', to='base.codeableconcept')),
                ('related', models.ManyToManyField(blank=True, related_name='document_reference_related', to='base.reference')),
                ('security_label', models.ManyToManyField(blank=True, related_name='document_reference_security_labels', to='base.codeableconcept')),
                ('subject', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_subject', to='base.reference')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_type', to='base.codeableconcept')),
                ('attester', models.ManyToManyField(blank=True, related_name='document_reference_attesters', to='documentreferences.documentreferenceattester')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""activity definition choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class ActivityDefinitionStatus(models.TextChoices):
    """Activity definition status."""

    DRAFT = "draft", _("Draft")
    ACTIVE = "active", _("Active")
    RETIRED = "retired", _("Retired")
    UNKNOWN = "unknown", _("Unknown")


class ActivityDefinitionKindChoices(models.TextChoices):
    """Activity definition kind choices."""

    APPOINTMENT = "appointment", _("Appointment")
    CARE_PLAN = "care-plan", _("Care Plan")
    CLAIM = "claim", _("Claim")
    COMMUNICATION_REQUEST = "communication-request", _("CommunicationRequest")
    COVERAGE_ELIGIBILITY_REQUEST = (
        "coverage-eligibility-request",
        _("CoverageEligibilityRequest"),
    )
    DEVICE_REQUEST = "device-request", _("DeviceRequest")
    ENROLLMENT_REQUEST = "enrollment-request", _("EnrollmentRequest")
    IMMUNIZATION_RECOMMENDATION = (
        "immunization-recommendation",
        _("ImmunizationRecommendation"),
    )
    MEDICATION_REQUEST = "medication-request", _("Medication Request")
    NUTRITION_ORDER = "nutrition-order", _("Nutrition Order")
    REQUEST_ORCHESTRATION = "request-orchestration", _("Request Orchestration")
    SERVICE_REQUEST = "service-request", _("Service Request")
    SUPPLY_REQUEST = "supply-request", _("Supply Request")
    TASK = "task", _("Task")
    TRANSPORT = "transport", _("Transport")
    VISION_PRESCRIPTION = "vision-prescription", _("Vision Prescription")


class ActivityDefinitionIntentChoices(models.TextChoices):
    """Activity definition intent choices."""

    PROPOSAL = "proposal", _("Proposal")
    PLAN = "plan", _("Plan")
    DIRECTIVE = "directive", _("Directive")
    ORDER = "order", _("Order")
    ORIGINAL_ORDER = "original-order", _("Original Order")
    REFLEX_ORDER = "reflex-order", _("Reflex Order")
    FILTER_ORDER = "filter-order", _("Filter Order")
    INSTANCE_ORDER = "instance-order", _("Instance Order")
    OPTION = "option", _("Option")


class ActivityDefinitionPriorityChoices(models.TextChoices):
    """Activity definition priority choices."""

    ROUTINE = "routine", _("Routine")
    URGENCY = "urgency", _("Urgency")
    ASAP = "asap", _("ASAP")
    STAT = "stat", _("STAT")


class ActivityDefinitionParticipantTypeChoices(models.TextChoices):
    """activity definition participant type choices."""

    CARETEAM = "careteam", _("Care Team")
    DEVICE = "device", _("Device")
    GROUP = "group", _("Group")
    HEALTHCARE_SERVICE = "healthcareservice", _("Healthcare Service")
    LOCATION = "location", _("Location")
    ORGANIZATION = "organization", _("Organization")
    PATIENT = "patient", _("Patient")
    PRACTITIONER = "practitioner", _("Practitioner")
    PRACTITIONER_ROLE = "practitionerrole", _("Practitioner Role")
    RELATED_PERSON = "relatedperson", _("Related Person")



"""Healthcare services serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.models import ServiceType
from dfhir.base.serializers import (
    AttachmentSerializer,
    AvailabilitySerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    ExtendedContactDetailSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    QuantitySerializer,
    RangeSerializer,
    ReferenceSerializer,
)
from dfhir.endpoints.serializers import EndpointSerializer
from dfhir.locations.serializers import LocationReferenceSerializer

from .models import (
    ClinicalSpecialty,
    HealthcareService,
    HealthCareServiceCodeableReference,
    HealthcareServiceEligibility,
    HealthCareServiceEligibilityValue,
    HealthCareServiceReference,
    ServiceCategory,
)


class HealthCareServiceReferenceSerializer(BaseReferenceModelSerializer):
    """HealthCareService reference serializer."""

    class Meta:
        """Meta class."""

        model = HealthCareServiceReference
        exclude = ["created_at", "updated_at"]


class HealthCareServiceCodeableReferenceSerializer(WritableNestedModelSerializer):
    """CodeableReference serializer."""

    reference = HealthCareServiceReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = HealthCareServiceCodeableReference
        exclude = ["created_at", "updated_at"]


class HealthcareServiceEligibilityValueSerializer(serializers.ModelSerializer):
    """Healthcare Service Eligibility Value serializer."""

    value_codeable_concept = CodeableConceptSerializer(required=False)
    value_quantity = QuantitySerializer(required=False)
    value_range = RangeSerializer(required=False)
    value_reference = ReferenceSerializer(required=False)

    class Meta:
        """Meta class."""

        model = HealthCareServiceEligibilityValue
        exclude = ["created_at", "updated_at"]


class HealthcareServiceEligibilitySerializer(WritableNestedModelSerializer):
    """Healthcare Service Eligibility serializer."""

    code = CodeableConceptSerializer(required=False)
    value = HealthcareServiceEligibilityValueSerializer(required=False)

    class Meta:
        """Meta class."""

        model = HealthcareServiceEligibility
        exclude = ["created_at", "updated_at"]


class HealthcareServiceSerializer(BaseWritableNestedModelSerializer):
    """Healthcare Service serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    offered_in = HealthCareServiceReferenceSerializer(many=True, required=False)
    location = LocationReferenceSerializer(many=True, required=False)
    contact = ExtendedContactDetailSerializer(many=True, required=False)
    provided_by = OrganizationReferenceSerializer(required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    specialty = CodeableConceptSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    coverage_area = LocationReferenceSerializer(many=True, required=False)
    service_provision_code = CodeableConceptSerializer(many=True, required=False)
    eligibility = HealthcareServiceEligibilitySerializer(many=True, required=False)
    characteristic = CodeableConceptSerializer(many=True, required=False)
    communication = CodeableConceptSerializer(many=True, required=False)
    availability = AvailabilitySerializer(many=False, required=False)
    program = CodeableConceptSerializer(many=True, required=False)
    photo = AttachmentSerializer(required=False)
    referral_method = CodeableConceptSerializer(many=True, required=False)
    endpoint = EndpointSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = HealthcareService

        exclude = ["created_at", "updated_at"]


class ServiceCategorySerializer(serializers.ModelSerializer):
    """Service Category serializer."""

    class Meta:
        """Meta class."""

        model = ServiceCategory
        exclude = ["created_at", "updated_at"]


class ClinicalSpecialtySerializer(serializers.ModelSerializer):
    """Clinical Specialty Valueset serializer."""

    class Meta:
        """Meta class."""

        model = ClinicalSpecialty
        exclude = ["created_at", "updated_at"]


class ServiceTypeSerializer(serializers.ModelSerializer):
    """Service Type serializer."""

    class Meta:
        """Meta class."""

        model = ServiceType
        exclude = ["created_at", "updated_at"]



"""custom migrate command to reload all models."""

from django.core.management import call_command
from django.core.management.base import BaseCommand
from django.db.migrations.state import ProjectState

# Save original methods in case you need fallback
original_reload_model = ProjectState.reload_model
original_reload_models = ProjectState.reload_models


def custom_reload_model(self, app_label, model_name, delay=False):
    """Custom reload model method to reload a single model."""
    if "apps" in self.__dict__:  # hasattr would cache the property
        self._reload(set([(app_label, model_name)]))


def custom_reload_models(self, models, delay=True):
    """Custom reload models method to reload multiple models."""
    if "apps" in self.__dict__:  # hasattr would cache the property
        related_models = set(self.models.keys()) if models is None else set()
        for app_label, model_name in models:
            related_models.update(self._find_reload_model(app_label, model_name, delay))
        self._reload(related_models)


# Apply monkey patches
ProjectState.reload_model = custom_reload_model
ProjectState.reload_models = custom_reload_models


class Command(BaseCommand):
    """Custom command to reload all models in the project."""

    help = "Prints 'hi' to the console"

    def handle(self, *args, **kwargs):
        """Handle method to execute the command."""
        # Call the custom command
        call_command("migrate", *args, **kwargs)



"""Explanation of Benefits URL Configuration."""

from django.urls import path

from . import views

app_name = "explanationofbenefits"


urlpatterns = [
    path("explanationofbenefits/", views.ExplanationOfBenefitListView.as_view()),
    path(
        "explanationofbenefits/<int:pk>/",
        views.ExplanationOfBenefitDetailView.as_view(),
    ),
]



"""Location admin."""

from django.contrib import admin

from .models import Location


@admin.register(Location)
class LocationAdmin(admin.ModelAdmin):
    """Location admin."""

    list_display = (
        "name",
        "status",
        "alias",
    )
    search_fields = (
        "name",
        "status",
        "alias",
    )



"""Nutrition products views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import NutritionProduct
from .serializers import NutritionProductSerializer


class NutritionProductListView(APIView):
    """Nutrition Product List View."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: NutritionProductSerializer(many=True)})
    def get(self, request):
        """Get all nutrition products."""
        nutrition_products = NutritionProduct.objects.all()
        serializer = NutritionProductSerializer(nutrition_products, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=NutritionProductSerializer, responses={201: NutritionProductSerializer}
    )
    def post(self, request):
        """Create a nutrition product."""
        serializer = NutritionProductSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class NutritionProductDetailView(APIView):
    """Nutrition Product Detail View."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get nutrition product object."""
        try:
            return NutritionProduct.objects.get(id=pk)
        except NutritionProduct.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: NutritionProductSerializer})
    def get(self, request, pk=None):
        """Get nutrition product detail."""
        nutrition_product = self.get_object(pk)
        serializer = NutritionProductSerializer(nutrition_product)
        return Response(serializer.data)

    @extend_schema(
        request=NutritionProductSerializer, responses={200: NutritionProductSerializer}
    )
    def patch(self, request, pk=None):
        """Update nutrition product."""
        nutrition_product = self.get_object(pk)
        serializer = NutritionProductSerializer(nutrition_product, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a nutrition product."""
        nutrition_product = self.get_object(pk)
        nutrition_product.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Vision prescription views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import VisionPrescription
from .serializers import VisionPrescriptionSerializer


class VisionPrescriptionListView(APIView):
    """Vision prescription list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: VisionPrescriptionSerializer(many=True)})
    def get(self, request):
        """Get vision prescriptions."""
        vision_prescriptions = VisionPrescription.objects.all()
        serializer = VisionPrescriptionSerializer(vision_prescriptions, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=VisionPrescriptionSerializer,
        responses={201: VisionPrescriptionSerializer},
    )
    def post(self, request):
        """Create a vision prescription."""
        serializer = VisionPrescriptionSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class VisionPrescriptionDetailView(APIView):
    """Vision prescription detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get vision prescription object."""
        try:
            return VisionPrescription.objects.get(pk=pk)
        except VisionPrescription.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: VisionPrescriptionSerializer})
    def get(self, request, pk=None):
        """Get a vision prescription."""
        queryset = self.get_object(pk)
        serializer = VisionPrescriptionSerializer(queryset)
        return Response(serializer.data)

    @extend_schema(responses={200: VisionPrescriptionSerializer})
    def patch(self, request, pk=None):
        """Update a vision prescription."""
        queryset = self.get_object(pk)
        serializer = VisionPrescriptionSerializer(
            queryset, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    def delete(self, request, pk=None):
        """Delete a vision prescription."""
        vision_prescription = self.get_object(pk)
        vision_prescription.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Adverse events serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
)
from dfhir.conditions.serializers import (
    ConditionObservationCodeableReferenceSerializer,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer

from .models import (
    AdverseEvent,
    AdverseEventContributingFactorCodeableReference,
    AdverseEventContributingFactorReference,
    AdverseEventMitigatingActionCodeableReference,
    AdverseEventMitigatingActionReference,
    AdverseEventParticipant,
    AdverseEventParticipantActorReference,
    AdverseEventPreventiveActionCodeableReference,
    AdverseEventPreventiveActionReference,
    AdverseEventRecorderReference,
    AdverseEventSubjectReference,
    AdverseEventSupportingInfoCodeableReference,
    AdverseEventSupportingInfoReference,
    AdverseEventSuspectEntity,
    AdverseEventSuspectEntityAuthorReference,
    AdverseEventSuspectEntityCausality,
    AdverseEventSuspectEntityInstanceCodeableReference,
    AdverseEventSuspectEntityInstanceReference,
)


class AdverseEventParticipantActorReferenceSerializer(BaseReferenceModelSerializer):
    """Adverse Event Participant Actor Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventParticipantActorReference
        exclude = ["created_at", "updated_at"]


class AdverseEventRecorderReferenceSerializer(BaseReferenceModelSerializer):
    """Adverse Event Recorder Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventRecorderReference
        exclude = ["created_at", "updated_at"]


class AdverseEventSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """Adverse Event Subject Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventSubjectReference
        exclude = ["created_at", "updated_at"]


class AdverseEventParticipantSerializer(BaseWritableNestedModelSerializer):
    """Adverse Event Participant Serializer."""

    actor = AdverseEventParticipantActorReferenceSerializer(many=False, required=False)
    function = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventParticipant
        exclude = ["created_at", "updated_at"]


class AdverseEventSuspectEntityInstanceReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Adverse Event Suspect Entity Instance Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventSuspectEntityInstanceReference
        exclude = ["created_at", "updated_at"]


class AdverseEventSuspectEntityInstanceCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """Adverse Event Suspect Entity Instance Codeable Reference Serializer."""

    reference = AdverseEventSuspectEntityInstanceReferenceSerializer(
        many=False, required=False
    )
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventSuspectEntityInstanceCodeableReference
        exclude = ["created_at", "updated_at"]


class AdverseEventSuspectEntityAuthorReferenceSerializer(BaseReferenceModelSerializer):
    """Adverse Event Suspect Entity Author Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventSuspectEntityAuthorReference
        exclude = ["created_at", "updated_at"]


class AdverseEventContributingFactorReferenceSerializer(BaseReferenceModelSerializer):
    """Adverse Event Contributing Factor Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventContributingFactorReference
        exclude = ["created_at", "updated_at"]


class AdverseEventContributingFactorCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """Adverse Event Contributing Factor Codeable Reference Serializer."""

    reference = AdverseEventContributingFactorReferenceSerializer(
        many=False, required=False
    )
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventContributingFactorCodeableReference
        exclude = ["created_at", "updated_at"]


class AdverseEventMitigatingActionReferenceSerializer(BaseReferenceModelSerializer):
    """Adverse Event Mitigating Action Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventMitigatingActionReference
        exclude = ["created_at", "updated_at"]


class AdverseEventMitigatingActionCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """Adverse Event Mitigating Action Codeable Reference Serializer."""

    reference = AdverseEventMitigatingActionReferenceSerializer(
        many=False, required=False
    )
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventMitigatingActionCodeableReference
        exclude = ["created_at", "updated_at"]


class AdverseEventPreventiveActionReferenceSerializer(BaseReferenceModelSerializer):
    """Adverse Event Preventive Action Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventPreventiveActionReference
        exclude = ["created_at", "updated_at"]


class AdverseEventPreventiveActionCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """Adverse Event Preventive Action Codeable Reference Serializer."""

    reference = AdverseEventPreventiveActionReferenceSerializer(
        many=False, required=False
    )
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventPreventiveActionCodeableReference
        exclude = ["created_at", "updated_at"]


class AdverseEventSupportingInfoReferenceSerializer(BaseReferenceModelSerializer):
    """Adverse Event Supporting Info Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventSupportingInfoReference
        exclude = ["created_at", "updated_at"]


class AdverseEventSupportingInfoCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """Adverse Event Supporting Info Codeable Reference Serializer."""

    reference = AdverseEventSupportingInfoReferenceSerializer(
        many=False, required=False
    )
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventSupportingInfoCodeableReference
        exclude = ["created_at", "updated_at"]


class AdverseEventSuspectEntityCausalitySerializer(BaseWritableNestedModelSerializer):
    """Adverse Event Suspect Entity Causality Serializer."""

    assessment_method = CodeableConceptSerializer(many=False, required=False)
    entity_relatedness = CodeableConceptSerializer(many=False, required=False)
    author = AdverseEventSuspectEntityAuthorReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = AdverseEventSuspectEntityCausality
        exclude = ["created_at", "updated_at"]


class AdverseEventSuspectEntitySerializer(BaseWritableNestedModelSerializer):
    """Adverse Event Suspect Entity Serializer."""

    instance = AdverseEventSuspectEntityInstanceCodeableReferenceSerializer(
        many=False, required=False
    )
    causality = AdverseEventSuspectEntityCausalitySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AdverseEventSuspectEntity
        exclude = ["created_at", "updated_at"]


class AdverseEventSerializer(BaseWritableNestedModelSerializer):
    """Adverse Event Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    subject = AdverseEventSubjectReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    cause_period = PeriodSerializer(many=False, required=False)
    effect_period = PeriodSerializer(many=False, required=False)
    resulting_effect = ConditionObservationCodeableReferenceSerializer(
        many=True, required=False
    )
    location = LocationReferenceSerializer(many=False, required=False)
    seriousness = CodeableConceptSerializer(many=False, required=False)
    outcome = CodeableConceptSerializer(many=True, required=False)
    recorder = AdverseEventRecorderReferenceSerializer(many=False, required=False)
    participant = AdverseEventParticipantSerializer(many=True, required=False)
    suspect_entity = AdverseEventSuspectEntitySerializer(many=True, required=False)
    contributing_factor = AdverseEventContributingFactorCodeableReferenceSerializer(
        many=True, required=False
    )
    preventive_action = AdverseEventPreventiveActionCodeableReferenceSerializer(
        many=True, required=False
    )
    mitigating_action = AdverseEventMitigatingActionCodeableReferenceSerializer(
        many=True, required=False
    )
    supporting_info = AdverseEventSupportingInfoCodeableReferenceSerializer(
        many=True, required=False
    )
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = AdverseEvent
        exclude = ["created_at", "updated_at"]



"""risk assessment views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.riskassessments.models import RiskAssessment
from dfhir.riskassessments.serializers import RiskAssessmentSerializer


class RiskManagementListView(APIView):
    """risk management list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: RiskAssessmentSerializer(many=True)})
    def get(self, request):
        """Get."""
        risk_assessments = RiskAssessment.objects.all()
        serializer = RiskAssessmentSerializer(risk_assessments, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=RiskAssessmentSerializer, responses={201: RiskAssessmentSerializer}
    )
    def post(self, request):
        """Post."""
        serializer = RiskAssessmentSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class RiskAssessmentDetialView(APIView):
    """risk assessment detail view."""

    def get_object(self, pk):
        """Get object."""
        try:
            return RiskAssessment.objects.get(pk=pk)
        except RiskAssessment.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: RiskAssessmentSerializer})
    def get(self, request, pk=None):
        """Get a risk assessment object by ID."""
        risk_assessment = self.get_object(pk)
        serializer = RiskAssessmentSerializer(risk_assessment)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=RiskAssessmentSerializer, responses={200: RiskAssessmentSerializer}
    )
    def patch(self, request, pk=None):
        """Update a risk assessment object by ID."""
        risk_assessment = self.get_object(pk)
        serializer = RiskAssessmentSerializer(
            risk_assessment, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a risk assessment object by ID."""
        risk_assessment = self.get_object(pk)
        risk_assessment.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""inventory item urls."""

from django.urls import path

from dfhir.inventoryitems.views import InventoryItemDetailView, InventoryItemListView

app_name = "inventoryitems"

urlpatterns = [
    path("inventoryitems/", InventoryItemListView.as_view(), name="inventoryitem-list"),
    path(
        "inventoryitems/<int:pk>/",
        InventoryItemDetailView.as_view(),
        name="inventoryitem-detail",
    ),
]



"""Patient-related choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class ContactRelationshipChoices(models.TextChoices):
    """Contact relationship choices."""

    BILLING_CONTACT_PERSON = "billing_contact_person", _("billing-contact-person")
    CONTACT_PERSON = "contact_person", _("contact-person")
    EMERGENCY_CONTACT_PERSON = "emergency_contact_person", _("emergency-contact-person")
    PERSON_PREPARING_REFERRAL = (
        "person_preparing_referral",
        _("person-preparing-referral"),
    )
    EMPLOYER = "employer", _("employer")
    EMERGENCY_CONTACT = "emergency_contact", _("emergency-contact")
    FEDERAL_AGENCY = "federal_agency", _("federal-agency")
    INSURANCE_COMPANY = "insurance_company", _("insurance-company")
    NEXT_OF_KIN = "next_of_kin", _("next-of-kin")
    STATE_AGENCY = "state_agency", _("state-agency")
    UNKNOWN = "unknown", _("unknown")


class PatientLinkChoices(models.TextChoices):
    """Patient link choices."""

    REPLACED_BY = "replaced_by", _("replaced-by")
    REPLACES = "replaces", _("replaces")
    REFER = "refer", _("refer")
    SEEALSO = "seealso", _("seealso")



"""immunization evaluations models."""

from django.db import models

from dfhir.base.models import Identifier, TimeStampedModel
from dfhir.immunizationevaluations.choices import ImmunizationEvaluationStatusChoices


class ImmunizationEvaluation(TimeStampedModel):
    """Immunization Evaluation model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="immunization_evaluation_identifier"
    )
    status = models.CharField(
        max_length=255, choices=ImmunizationEvaluationStatusChoices.choices, null=True
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_evaluation_patient",
    )
    date = models.DateTimeField(null=True)
    authority = models.ForeignKey(
        "base.OrganizationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_evaluation_authority",
    )
    target_disease = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_evaluation_target_disease",
    )
    immunization_event = models.ForeignKey(
        "immunizations.ImmunizationReference",
        on_delete=models.DO_NOTHING,
        related_name="immunization_evaluation_immunization_event",
        null=True,
    )
    dose_status = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="immunization_evaluation_dose_status",
    )
    dose_status_reason = models.ManyToManyField(
        "base.CodeableConcept",
        related_name="immunization_evaluation_dose_status_reason",
        blank=True,
    )
    description = models.TextField(null=True)
    series = models.CharField(max_length=255, null=True)
    dose_number = models.CharField(max_length=255, null=True)
    series_doses = models.CharField(max_length=255, null=True)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('encounters', '0001_initial'),
        ('episodeofcares', '0001_initial'),
        ('healthcareservices', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='encounter',
            name='episode_of_care',
            field=models.ManyToManyField(blank=True, related_name='encounter_episode_of_care', to='episodeofcares.episodeofcare'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='encounter_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='klass',
            field=models.ManyToManyField(blank=True, related_name='encounter_class', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='length',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_length', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='priority',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_priority', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='service_provider',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_service_provider', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='service_type',
            field=models.ManyToManyField(related_name='encounter_service_type', to='healthcareservices.healthcareservicecodeablereference'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='special_arrangement',
            field=models.ManyToManyField(blank=True, related_name='encounter_special_arrangement', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='special_courtesy',
            field=models.ManyToManyField(blank=True, related_name='encounter_special_courtesy', to='base.codeableconcept'),
        ),
    ]



"""specimens models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Duration,
    Identifier,
    SimpleQuantity,
    TimeStampedModel,
)
from dfhir.devices.models import DeviceCodeableReference
from dfhir.procedures.models import ProcedureReference
from dfhir.specimens.choices import SpecimenCombinedChoices, SpecimenStatus
from dfhir.substances.models import SubstanceReference


class SpecimenReference(BaseReference):
    """Specimen reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_reference_identifier",
    )
    specimen = models.ForeignKey(
        "Specimen",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_reference_specimen",
    )


class SpecimenFeature(TimeStampedModel):
    """Specimen feature model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_feature_type",
    )
    description = models.TextField(null=True)


class SpecimenCollectorReference(BaseReference):
    """Specimen collector reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_collector_reference_identifier",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="specimen_collector_reference_organization",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="specimen_collector_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="specimen_collector_reference_practitioner_role",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="specimen_collector_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="specimen_collector_reference_related_person",
        null=True,
    )


class SpecimenCollection(TimeStampedModel):
    """Specimen collections model."""

    collector = models.ForeignKey(
        SpecimenCollectorReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_collections_collector",
    )
    collected_date = models.DateTimeField(null=True)
    collected_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        related_name="specimen_collections_collected_period",
        null=True,
    )
    duration = models.ForeignKey(
        Duration,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_collections_duration",
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.DO_NOTHING,
        related_name="specimen_collections_method",
        null=True,
    )
    method = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="specimen_collections_method",
        null=True,
    )
    device = models.ForeignKey(
        DeviceCodeableReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_collections_device",
    )
    procedure = models.ForeignKey(
        ProcedureReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_collections_procedure",
    )
    body_site = models.ForeignKey(
        "bodystructures.BodyStructureCodeableReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_collections_body_site",
    )
    fasting_status_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_collections_fasting_status_codeable_concept",
    )
    fasting_status_duration = models.ForeignKey(
        Duration,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_collections_fasting_status_duration",
    )


class SpecimenProcessingPerformerReference(BaseReference):
    """Specimen performer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_performer_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_performer_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_performer_reference_practitioner_role",
    )

    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_performer_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_performer_reference_related_person",
    )


class SpecimenProcessing(TimeStampedModel):
    """Specimen processing model."""

    description = models.TextField(null=True)
    method = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_processing_method",
    )
    performer = models.ForeignKey(
        SpecimenProcessingPerformerReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_processing_performer",
    )
    device = models.ForeignKey(
        "devices.DeviceReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_processing_device",
    )
    additive = models.ManyToManyField(
        SubstanceReference, blank=True, related_name="specimen_processing_additive"
    )
    time_date_time = models.DateTimeField(null=True)
    time_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_processing_time_period",
    )
    time_duration = models.ForeignKey(
        "base.Duration",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_processing_time_duration",
    )


class SpecimenContainer(TimeStampedModel):
    """Specimen container model."""

    device = models.ForeignKey(
        "devices.DeviceReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_container_device",
    )
    specimen_quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_container_specimen_quantity",
    )


class SpecimenSubjectReference(BaseReference):
    """Specimen subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_subject_reference_identifier",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_subject_reference_patient",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_subject_reference_group",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_subject_reference_device",
    )
    biologically_driven_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_subject_reference_biologically_driven_product",
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="specimen_subject_reference_substance",
    )
    location = models.ForeignKey(
        "locations.Location",
        related_name="specimen_subject_reference_location",
        on_delete=models.DO_NOTHING,
        null=True,
    )


class Specimen(TimeStampedModel):
    """Specimen model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="specimen_identifier", blank=True
    )
    accession_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="specimen_accession_identifier",
        null=True,
    )
    status = models.CharField(max_length=255, null=True, choices=SpecimenStatus.choices)
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="specimen_type",
        null=True,
    )
    subject = models.ForeignKey(
        SpecimenSubjectReference,
        on_delete=models.DO_NOTHING,
        related_name="specimen_subject",
    )
    received_time = models.DateTimeField(null=True)
    parent = models.ManyToManyField(
        SpecimenReference, related_name="specimen_parent", blank=True
    )
    request = models.ManyToManyField(
        "servicerequests.ServiceRequestReference",
        related_name="specimen_request",
        blank=True,
    )
    combined = models.CharField(
        max_length=255, null=True, choices=SpecimenCombinedChoices.choices
    )
    role = models.ManyToManyField(
        CodeableConcept, related_name="specimen_role", blank=True
    )
    feature = models.ManyToManyField(
        SpecimenFeature, related_name="specimen_feature", blank=True
    )
    collections = models.ForeignKey(
        SpecimenCollection,
        on_delete=models.DO_NOTHING,
        related_name="specimen_collections",
        null=True,
    )
    processing = models.ManyToManyField(
        SpecimenProcessing, related_name="specimen_processing", blank=True
    )
    container = models.ManyToManyField(
        SpecimenContainer,
        related_name="specimen_container",
        blank=True,
    )
    condition = models.ManyToManyField(
        CodeableConcept, related_name="specimen_condition", blank=True
    )
    note = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="specimen_note"
    )



"""observation definition model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class QualifiedValueGenderChoices(TextChoices):
    """qualified value gender choices."""

    MALE = "male", _("Male")
    FEMALE = "female", _("Female")
    OTHER = "other", _("Other")
    UNKNOWN = "unknown", _("Unknown")


class QualifiedValueRangeCategoryChoices(TextChoices):
    """qualified value range category choices."""

    REFERENCE = "reference", _("Reference")
    CRITICAL = "critical", _("Critical")
    ABSOLUTE = "absolute", _("Absolute")


class ObservationDefinitionStatusChoices(TextChoices):
    """observation definition status choices."""

    DRAFT = "draft", _("Draft")
    ACTIVE = "active", _("Active")
    RETIRED = "retired", _("Retired")
    UNKNOWN = "unknown", _("Unknown")


class ObservationDefinitionPermittedDataTypeChoices(TextChoices):
    """observation definition permitted data type choices."""

    QUANTITY = "Quantity", _("Quantity")
    CODEABLE_CONCEPT = "codeableconcept", _("Codeable Concept")
    STRING = "string", _("String")
    BOOLEAN = "boolean", _("Boolean")
    INTEGER = "integer", _("Integer")
    RANGE = "range", _("Range")
    RATIO = "ratio", _("Ratio")
    SAMPLED_DATA = "sampled_data", _("Sampled Data")
    TIME = "time", _("Time")
    DATE = "date", _("Date")
    DATETIME = "datetime", _("Datetime")
    PERIOD = "period", _("Period")



"""Device Metric Calibration model."""

from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import DeviceMetric
from .serializers import DeviceMetricSerializer


class DeviceMetricListView(APIView):
    """Device Metric list view."""

    serializer = DeviceMetricSerializer
    permission_classes = [AllowAny]

    @extend_schema(responses={200, DeviceMetricSerializer(many=True)})
    def get(self, request):
        """Get all device metrics."""
        device_metrics = DeviceMetric.objects.all()
        serializer = self.serializer(device_metrics, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=DeviceMetricSerializer,
        responses={201, DeviceMetricSerializer},
    )
    def post(self, request):
        """Create a new device metric."""
        serializer = self.serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class DeviceMetricDetailView(APIView):
    """Device Metric detail view."""

    serializer = DeviceMetricSerializer

    def get_object(self, pk=None):
        """Get device metric object."""
        try:
            return DeviceMetric.objects.get(pk=pk)
        except DeviceMetric.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)

    @extend_schema(responses={200, DeviceMetricSerializer})
    def get(self, request, pk):
        """Get a device metric."""
        device_metric = self.get_object(pk)
        serializer = self.serializer(device_metric)
        return Response(serializer.data)

    @extend_schema(responses={200, DeviceMetricSerializer})
    def patch(self, request, pk):
        """Update a device metric."""
        device_metric = self.get_object(pk)
        serializer = self.serializer(device_metric, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, pk):
        """Delete a device metric."""
        device_metric = self.get_object(pk)
        device_metric.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""care plan app."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class CareplansConfig(AppConfig):
    """Care Plans Config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.careplans"
    verbose_name = _("Care Plans")



"""Claims urls."""

from django.urls import path

from . import views

app_name = "claims"

urlpatterns = [
    path("claims/", views.ClaimListView.as_view()),
    path("claims/<int:pk>/", views.ClaimDetailView.as_view()),
]



"""Choices for EpisodeOfCare model."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class EpisodeOfCareStatusChoices(models.TextChoices):
    """Choices for EpisodeOfCare status."""

    PLANNED = "planned", _("Planned")
    WAITLIST = "waitlist", _("Waitlist")
    ACTIVE = "active", _("Active")
    ONHOLD = "onhold", _("On Hold")
    FINISHED = "finished", _("Finished")
    CANCELLED = "cancelled", _("Cancelled")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")



"""service requests views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.generics import ListAPIView
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import (
    AsNeeded,
    BodySite,
    Parameter,
    ProcedureCodes,
    ProcedureReason,
    Reference,
    ServiceRequest,
    ServiceRequestCategory,
)
from .serializers import (
    AsNeededSerializer,
    BodySiteSerializer,
    ParameterSerializer,
    ProcedureCodeSerializer,
    ProcedureReasonSerializer,
    ReferenceSerializer,
    ServiceRequestCategorySerializer,
    ServiceRequestSerializer,
)


class ServiceRequestListView(APIView):
    """service request list views."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200, ServiceRequestSerializer(many=True)})
    def get(self, request):
        """Get all service requests."""
        service_request = ServiceRequest.objects.all()
        serializer = ServiceRequestSerializer(service_request, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ServiceRequestSerializer, responses={201, ServiceRequestSerializer}
    )
    def post(self, request):
        """Service request create view."""
        serializer = ServiceRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ServiceRequestDetailView(APIView):
    """service request detail views."""

    def get_object(self, pk):
        """Get object function."""
        try:
            return ServiceRequest.objects.get(pk=pk)
        except ServiceRequest.DoesNotExist as error:
            raise Http404 from error

    @extend_schema(responses={200, ServiceRequestSerializer})
    def get(self, request, pk=None):
        """Service request retrieve view."""
        service_request = self.get_object(pk)
        serializer = ServiceRequestSerializer(service_request)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200, ServiceRequestSerializer})
    def patch(self, request, pk=None):
        """Service request update view."""
        service_request = self.get_object(pk)
        serializer = ServiceRequestSerializer(
            service_request, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Service reequest delete view."""
        service_request = self.get_object(pk)
        service_request.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


@extend_schema(responses={200, ParameterSerializer})
class ParameterListView(ListAPIView):
    """parameter list view."""

    serializer_class = ParameterSerializer
    permission_classes = [AllowAny]
    queryset = Parameter.objects.all()


@extend_schema(responses={200, ReferenceSerializer})
class ReferenceListView(ListAPIView):
    """reference list view."""

    serializer_class = ReferenceSerializer
    permission_classes = [AllowAny]
    queryset = Reference.objects.all()


@extend_schema(responses={200, ServiceRequestCategorySerializer})
class ServiceRequestCategoryListView(ListAPIView):
    """service request list view."""

    serializer_class = ServiceRequestCategorySerializer
    permission_classes = [AllowAny]
    queryset = ServiceRequestCategory.objects.all()


@extend_schema(responses={200, AsNeededSerializer})
class AsNeededListView(ListAPIView):
    """as needed list view."""

    permission_classes = [AllowAny]
    serializer_class = AsNeededSerializer
    queryset = AsNeeded.objects.all()


@extend_schema(responses={200, ProcedureReasonSerializer})
class ProcedureReasonListView(ListAPIView):
    """procedure reason list view."""

    serializer_class = ProcedureReasonSerializer
    permission_classes = [AllowAny]
    queryset = ProcedureReason.objects.all()


@extend_schema(responses={200, BodySiteSerializer})
class BodySiteListView(ListAPIView):
    """body site list view."""

    permission_classes = [AllowAny]
    serializer_class = BodySiteSerializer
    queryset = BodySite.objects.all()


@extend_schema(responses={200, ProcedureCodeSerializer})
class ProcedureCodesListView(ListAPIView):
    """procedure codes list view."""

    permission_classes = [AllowAny]
    serializer_class = ProcedureCodeSerializer
    queryset = ProcedureCodes.objects.all()



"""image selection models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)

from .choices import (
    ImagingSelectionInstanceImageRegion2DRegionTypeChoices,
    ImagingSelectionInstanceImageRegion3DRegionTypeChoices,
    ImagingSelectionStatusChoices,
)

# Create your models here.


class ImagingSelectionPerformerActorReference(BaseReference):
    """imaging selection actor reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_practitioner_role",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_device",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_organization",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeamReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_care_team",
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_related_person",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_actor_reference_healthcare_service",
    )


class ImagingSelectionPerformer(TimeStampedModel):
    """imaging selection performer model."""

    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="imaging_selection_performer_function",
        null=True,
    )
    actor = models.ForeignKey(
        ImagingSelectionPerformerActorReference,
        on_delete=models.DO_NOTHING,
        related_name="imaging_selection_performer_actor",
        null=True,
    )


class ImagingSelectionBasedOnReference(BaseReference):
    """image selection based on reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_based_on_reference_identifier",
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_based_on_reference_care_plan",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_based_on_reference_service_request",
    )
    appointment = models.ForeignKey(
        "appointments.Appointment",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_based_on_reference_appointment",
    )
    task = models.ForeignKey(
        "tasks.Task",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_based_on_reference_task",
    )


class ImagingSelectionInstanceImagingRegion2D(TimeStampedModel):
    """image selection instance imaging region 2d model."""

    region_type = models.CharField(
        max_length=255,
        null=True,
        choices=ImagingSelectionInstanceImageRegion2DRegionTypeChoices.choices,
    )
    coordinate = ArrayField(models.FloatField(null=True), null=True)


class ImagingSelectionInstance(TimeStampedModel):
    """imaging selection instance model."""

    uid = models.CharField(max_length=255, null=True)
    number = models.IntegerField(null=True)
    sop_class = models.CharField(max_length=255, null=True)
    subset = ArrayField(models.CharField(max_length=255, null=True), null=True)
    imaging_region_2d = models.ManyToManyField(
        ImagingSelectionInstanceImagingRegion2D,
        blank=True,
        related_name="image_selection_instance_imaging_region_2d",
    )


class ImagingSelectionImageRegion3D(TimeStampedModel):
    """image selection image region 3d model."""

    region_type = models.CharField(
        max_length=255,
        null=True,
        choices=ImagingSelectionInstanceImageRegion3DRegionTypeChoices.choices,
    )
    coordinate = ArrayField(models.FloatField(null=True), null=True)


class ImagingSelectionSubjectReference(BaseReference):
    """imaging selection subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_identifier",
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_patient",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_group",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_device",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_procedure",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_practitioner",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_medication",
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_substance",
    )
    specimen = models.ForeignKey(
        "specimens.Specimen",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_selection_subject_reference_specimen",
    )


class ImagingSelection(TimeStampedModel):
    """image selection model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="image_selection_identifier"
    )
    status = models.CharField(
        max_length=255, null=True, choices=ImagingSelectionStatusChoices.choices
    )
    category = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="image_selection_category"
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_code",
    )
    subject = models.ForeignKey(
        ImagingSelectionSubjectReference,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="image_selection_subject",
    )
    issued = models.DateTimeField(null=True)
    performer = models.ManyToManyField(
        ImagingSelectionPerformer, blank=True, related_name="image_selection_performer"
    )
    based_on = models.ManyToManyField(
        ImagingSelectionBasedOnReference,
        blank=True,
        related_name="image_selection_based_on",
    )
    derived_from = models.ForeignKey(
        "imagingstudies.ImagingStudyReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_derived_from",
    )
    study_uid = models.CharField(max_length=255, null=True)
    series_uid = models.CharField(max_length=255, null=True)
    series_number = models.IntegerField(null=True)
    frame_of_reference_uid = models.CharField(max_length=255, null=True)
    body_site = models.ManyToManyField(
        "bodystructures.BodyStructureCodeableReference",
        blank=True,
        related_name="imaging_selection_bodysite",
    )
    focus = models.ManyToManyField(
        "ImagingSelectionReference", blank=True, related_name="imaging_selection_focus"
    )
    endpoint = models.ManyToManyField(
        "endpoints.EndpointReference",
        blank=True,
        related_name="imaging_selection_endpoint",
    )
    instance = models.ManyToManyField(
        ImagingSelectionInstance, blank=True, related_name="image_selection_instance"
    )
    image_region_3d = models.ManyToManyField(
        ImagingSelectionImageRegion3D,
        blank=True,
        related_name="image_selection_image_region_3d",
    )


class ImagingSelectionReference(BaseReference):
    """image selection reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_reference_identifier",
    )
    image_selection = models.ForeignKey(
        ImagingSelection,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_reference_image_selection",
    )


class ImagingSelectionCodeableReference(TimeStampedModel):
    """image selection codeable reference model."""

    concept = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        ImagingSelectionReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="image_selection_codeable_reference_reference",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('medicationadministrations', '0002_initial'),
        ('medicationdispenses', '0002_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationadministration',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='medicationadministration',
            name='supporting_information',
            field=models.ManyToManyField(related_name='medication_administration_supporting_information', to='base.reference'),
        ),
        migrations.AddField(
            model_name='medicationadministrationdosage',
            name='dose',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_dosage_dose', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='medicationadministrationdosage',
            name='method',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_dosage_method', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationadministrationdosage',
            name='rate_quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_dosage_rate_quantity', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='medicationadministrationdosage',
            name='rate_ratio',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_dosage_rate_ratio', to='base.ratio'),
        ),
        migrations.AddField(
            model_name='medicationadministrationdosage',
            name='route',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_dosage_route', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationadministrationdosage',
            name='site',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_dosage_site', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationadministration',
            name='dosage',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_dosage', to='medicationadministrations.medicationadministrationdosage'),
        ),
        migrations.AddField(
            model_name='medicationadministrationpartof',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='medication_administration_part_of_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationadministrationpartof',
            name='medication_administraton',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_part_of_medication_administration', to='medicationadministrations.medicationadministration'),
        ),
        migrations.AddField(
            model_name='medicationadministrationpartof',
            name='medication_dispense',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_part_of_medication_dispense', to='medicationdispenses.medicationdispense'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
        ('endpoints', '0001_initial'),
        ('locations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClinicalSpecialty',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ServiceCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ServiceType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
        ),
        migrations.CreateModel(
            name='HealthcareServiceEligibility',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('comment', models.TextField(null=True)),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='healthcareserviceeligibility_code', to='base.codeableconcept')),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='HealthcareService',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('name', models.CharField(max_length=255)),
                ('comment', models.TextField(null=True)),
                ('extra_details', models.TextField(null=True)),
                ('referral_required', models.BooleanField(default=False)),
                ('appointment_required', models.BooleanField(default=False)),
                ('availability', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='healthcareservice_availability', to='base.availability')),
                ('category', models.ManyToManyField(blank=True, related_name='healthcareservice_category', to='base.codeableconcept')),
                ('characteristic', models.ManyToManyField(blank=True, related_name='healthcareservice_characteristics', to='base.codeableconcept')),
                ('communication', models.ManyToManyField(blank=True, related_name='healthcareservice_communication', to='base.codeableconcept')),
                ('contact', models.ManyToManyField(blank=True, related_name='healthcareservice_contact', to='base.extendedcontactdetail')),
                ('coverage_area', models.ManyToManyField(blank=True, related_name='healthcareservice_coverage_area', to='locations.locationreference')),
                ('endpoint', models.ManyToManyField(blank=True, related_name='healthcareservice_endpoint', to='endpoints.endpoint')),
                ('identifier', models.ManyToManyField(blank=True, related_name='healthcareservice_identifier', to='base.identifier')),
                ('location', models.ManyToManyField(blank=True, related_name='healthcare_service_location', to='locations.locationreference')),
                ('photo', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='base.attachment')),
                ('program', models.ManyToManyField(blank=True, related_name='healthcareservice_program', to='base.codeableconcept')),
                ('provided_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.organizationreference')),
                ('referral_method', models.ManyToManyField(blank=True, related_name='healthcareservice_referral_method', to='base.codeableconcept')),
                ('service_provision_code', models.ManyToManyField(blank=True, related_name='healthcareservice_service_provision_code', to='base.codeableconcept')),
                ('specialty', models.ManyToManyField(blank=True, related_name='healthcareservice_specialty', to='base.codeableconcept')),
                ('type', models.ManyToManyField(blank=True, related_name='healthcareservice_type', to='base.codeableconcept')),
                ('eligibility', models.ManyToManyField(blank=True, related_name='healthcareservice_eligibility', to='healthcareservices.healthcareserviceeligibility')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='HealthCareServiceEligibilityValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.codeableconcept')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.quantity')),
                ('value_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.range')),
                ('value_reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.reference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='healthcareserviceeligibility',
            name='value',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='healthcareservices.healthcareserviceeligibilityvalue'),
        ),
        migrations.CreateModel(
            name='HealthCareServiceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('healthcare_service', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='healthcareservice_reference', to='healthcareservices.healthcareservice')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='healthcareservice_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='HealthCareServiceCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='healthcareservice_codeableconcept', to='base.codeableconcept')),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='codeable_reference', to='healthcareservices.healthcareservicereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='healthcareservice',
            name='offered_in',
            field=models.ManyToManyField(blank=True, related_name='healthcareservice_offered_in', to='healthcareservices.healthcareservicereference'),
        ),
    ]



"""formulary item models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)
from dfhir.formularyitems.choices import FormularyItemStatusChoices


class FormularyItem(TimeStampedModel):
    """formulary item model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="formulary_item_identifier"
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="formulary_item_code",
    )
    status = models.CharField(
        max_length=200, null=True, choices=FormularyItemStatusChoices.choices
    )


class FormularyItemReference(BaseReference):
    """formulary item reference model."""

    identifier = models.ForeignKey(
        Identifier,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="formulary_item_reference_identifier",
    )
    formulary_item = models.ForeignKey(
        FormularyItem,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="formulary_item_reference_formulary_item",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
        ('devicemetrics', '0001_initial'),
        ('devices', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='devicemetric',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_metric_device', to='devices.devicereference'),
        ),
        migrations.AddField(
            model_name='devicemetric',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='device_metrics_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='devicemetric',
            name='measurement_frequency',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_metric_measurement_frequency', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='devicemetric',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_metric_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='devicemetric',
            name='unit',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_metric_unit', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='devicemetriccalibration',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_metric_calibration_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='devicemetric',
            name='calibration',
            field=models.ManyToManyField(blank=True, related_name='device_metric_calibration', to='devicemetrics.devicemetriccalibration'),
        ),
        migrations.AddField(
            model_name='devicemetricreference',
            name='device_metric',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_metric_reference', to='devicemetrics.devicemetric'),
        ),
        migrations.AddField(
            model_name='devicemetricreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_metric_reference_identifier', to='base.identifier'),
        ),
    ]



# Generated by Django 5.2 on 2025-05-06 15:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0004_initial'),
        ('coverages', '0002_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='accountcoverage',
            name='coverage',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_coverage_coverage', to='coverages.coveragereference'),
        ),
    ]
